@@ -3,85 +3,108 @@
 #include "duckdb/common/atomic.hpp"
 #include "duckdb/common/exception/http_exception.hpp"
 #include "duckdb/common/file_opener.hpp"
-#include "duckdb/common/http_state.hpp"
+#include "duckdb/common/helper.hpp"
+#include "duckdb/common/http_util.hpp"
 #include "duckdb/common/thread.hpp"
 #include "duckdb/common/types/hash.hpp"
 #include "duckdb/function/scalar/strftime_format.hpp"
 #include "duckdb/logging/http_logger.hpp"
 #include "duckdb/main/client_context.hpp"
 #include "duckdb/main/database.hpp"
-#include "duckdb/common/helper.hpp"
 #include "duckdb/main/secret/secret_manager.hpp"
+#include "http_state.hpp"
 
 #include <chrono>
+#include <map>
 #include <string>
 #include <thread>
 
 #define CPPHTTPLIB_OPENSSL_SUPPORT
 #include "httplib.hpp"
 
-#include <map>
-
 namespace duckdb {
 
-static duckdb::unique_ptr<duckdb_httplib_openssl::Headers> initialize_http_headers(HeaderMap &header_map) {
+duckdb::unique_ptr<duckdb_httplib_openssl::Headers> HTTPFileSystem::InitializeHeaders(HeaderMap &header_map,
+                                                                                      const HTTPParams &http_params) {
 	auto headers = make_uniq<duckdb_httplib_openssl::Headers>();
 	for (auto &entry : header_map) {
 		headers->insert(entry);
 	}
+
+	for (auto &entry : http_params.extra_headers) {
+		headers->insert(entry);
+	}
+
 	return headers;
 }
 
-HTTPParams HTTPParams::ReadFrom(optional_ptr<FileOpener> opener) {
-	uint64_t timeout = DEFAULT_TIMEOUT;
-	uint64_t retries = DEFAULT_RETRIES;
-	uint64_t retry_wait_ms = DEFAULT_RETRY_WAIT_MS;
-	float retry_backoff = DEFAULT_RETRY_BACKOFF;
-	bool force_download = DEFAULT_FORCE_DOWNLOAD;
-	bool keep_alive = DEFAULT_KEEP_ALIVE;
-	bool enable_server_cert_verification = DEFAULT_ENABLE_SERVER_CERT_VERIFICATION;
-	std::string ca_cert_file;
-	uint64_t hf_max_per_page = DEFAULT_HF_MAX_PER_PAGE;
+HTTPParams HTTPParams::ReadFrom(optional_ptr<FileOpener> opener, optional_ptr<FileOpenerInfo> info) {
+	auto result = HTTPParams();
+
+	// No point in continueing without an opener
+	if (!opener) {
+		return result;
+	}
 
 	Value value;
-	if (FileOpener::TryGetCurrentSetting(opener, "http_timeout", value)) {
-		timeout = value.GetValue<uint64_t>();
+
+	// Setting lookups
+	FileOpener::TryGetCurrentSetting(opener, "http_timeout", result.timeout, info);
+	FileOpener::TryGetCurrentSetting(opener, "force_download", result.force_download, info);
+	FileOpener::TryGetCurrentSetting(opener, "http_retries", result.retries, info);
+	FileOpener::TryGetCurrentSetting(opener, "http_retry_wait_ms", result.retry_wait_ms, info);
+	FileOpener::TryGetCurrentSetting(opener, "http_retry_backoff", result.retry_backoff, info);
+	FileOpener::TryGetCurrentSetting(opener, "http_keep_alive", result.keep_alive, info);
+	FileOpener::TryGetCurrentSetting(opener, "enable_server_cert_verification", result.enable_server_cert_verification,
+	                                 info);
+	FileOpener::TryGetCurrentSetting(opener, "ca_cert_file", result.ca_cert_file, info);
+	FileOpener::TryGetCurrentSetting(opener, "hf_max_per_page", result.hf_max_per_page, info);
+
+	// HTTP Secret lookups
+	KeyValueSecretReader settings_reader(*opener, info, "http");
+
+	string proxy_setting;
+	if (settings_reader.TryGetSecretKeyOrSetting<string>("http_proxy", "http_proxy", proxy_setting) &&
+	    !proxy_setting.empty()) {
+		idx_t port;
+		string host;
+		HTTPUtil::ParseHTTPProxyHost(proxy_setting, host, port);
+		result.http_proxy = host;
+		result.http_proxy_port = port;
 	}
-	if (FileOpener::TryGetCurrentSetting(opener, "force_download", value)) {
-		force_download = value.GetValue<bool>();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "http_retries", value)) {
-		retries = value.GetValue<uint64_t>();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "http_retry_wait_ms", value)) {
-		retry_wait_ms = value.GetValue<uint64_t>();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "http_retry_backoff", value)) {
-		retry_backoff = value.GetValue<float>();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "http_keep_alive", value)) {
-		keep_alive = value.GetValue<bool>();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "enable_server_cert_verification", value)) {
-		enable_server_cert_verification = value.GetValue<bool>();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "ca_cert_file", value)) {
-		ca_cert_file = value.ToString();
-	}
-	if (FileOpener::TryGetCurrentSetting(opener, "hf_max_per_page", value)) {
-		hf_max_per_page = value.GetValue<uint64_t>();
+	settings_reader.TryGetSecretKeyOrSetting<string>("http_proxy_username", "http_proxy_username",
+	                                                 result.http_proxy_username);
+	settings_reader.TryGetSecretKeyOrSetting<string>("http_proxy_password", "http_proxy_password",
+	                                                 result.http_proxy_password);
+	settings_reader.TryGetSecretKey<string>("bearer_token", result.bearer_token);
+
+	Value extra_headers;
+	if (settings_reader.TryGetSecretKey("extra_http_headers", extra_headers)) {
+		auto children = MapValue::GetChildren(extra_headers);
+		for (const auto &child : children) {
+			auto kv = StructValue::GetChildren(child);
+			D_ASSERT(kv.size() == 2);
+			result.extra_headers[kv[0].GetValue<string>()] = kv[1].GetValue<string>();
+		}
 	}
 
-	return {timeout,
-	        retries,
-	        retry_wait_ms,
-	        retry_backoff,
-	        force_download,
-	        keep_alive,
-	        enable_server_cert_verification,
-	        ca_cert_file,
-	        "",
-	        hf_max_per_page};
+	return result;
+}
+
+unique_ptr<duckdb_httplib_openssl::Client> HTTPClientCache::GetClient() {
+	lock_guard<mutex> lck(lock);
+	if (clients.size() == 0) {
+		return nullptr;
+	}
+
+	auto client = std::move(clients.back());
+	clients.pop_back();
+	return client;
+}
+
+void HTTPClientCache::StoreClient(unique_ptr<duckdb_httplib_openssl::Client> client) {
+	lock_guard<mutex> lck(lock);
+	clients.push_back(std::move(client));
 }
 
 void HTTPFileSystem::ParseUrl(string &url, string &path_out, string &proto_host_port_out) {
@@ -165,18 +188,18 @@ HTTPFileSystem::RunRequestWithRetry(const std::function<duckdb_httplib_openssl::
 unique_ptr<ResponseWrapper> HTTPFileSystem::PostRequest(FileHandle &handle, string url, HeaderMap header_map,
                                                         duckdb::unique_ptr<char[]> &buffer_out, idx_t &buffer_out_len,
                                                         char *buffer_in, idx_t buffer_in_len, string params) {
-	auto &hfs = handle.Cast<HTTPFileHandle>();
+	auto &hfh = handle.Cast<HTTPFileHandle>();
 	string path, proto_host_port;
 	ParseUrl(url, path, proto_host_port);
-	auto headers = initialize_http_headers(header_map);
+	auto headers = InitializeHeaders(header_map, hfh.http_params);
 	idx_t out_offset = 0;
 
 	std::function<duckdb_httplib_openssl::Result(void)> request([&]() {
-		auto client = GetClient(hfs.http_params, proto_host_port.c_str(), &hfs);
+		auto client = GetClient(hfh.http_params, proto_host_port.c_str(), &hfh);
 
-		if (hfs.state) {
-			hfs.state->post_count++;
-			hfs.state->total_bytes_sent += buffer_in_len;
+		if (hfh.state) {
+			hfh.state->post_count++;
+			hfh.state->total_bytes_sent += buffer_in_len;
 		}
 
 		// We use a custom Request method here, because there is no Post call with a contentreceiver in httplib
@@ -187,8 +210,8 @@ unique_ptr<ResponseWrapper> HTTPFileSystem::PostRequest(FileHandle &handle, stri
 		req.headers.emplace("Content-Type", "application/octet-stream");
 		req.content_receiver = [&](const char *data, size_t data_length, uint64_t /*offset*/,
 		                           uint64_t /*total_length*/) {
-			if (hfs.state) {
-				hfs.state->total_bytes_received += data_length;
+			if (hfh.state) {
+				hfh.state->total_bytes_received += data_length;
 			}
 			if (out_offset + data_length > buffer_out_len) {
 				// Buffer too small, increase its size by at least 2x to fit the new value
@@ -206,12 +229,12 @@ unique_ptr<ResponseWrapper> HTTPFileSystem::PostRequest(FileHandle &handle, stri
 		return client->send(req);
 	});
 
-	return RunRequestWithRetry(request, url, "POST", hfs.http_params);
+	return RunRequestWithRetry(request, url, "POST", hfh.http_params);
 }
 
 unique_ptr<duckdb_httplib_openssl::Client> HTTPFileSystem::GetClient(const HTTPParams &http_params,
                                                                      const char *proto_host_port,
-                                                                     optional_ptr<HTTPFileHandle> hfs) {
+                                                                     optional_ptr<HTTPFileHandle> hfh) {
 	auto client = make_uniq<duckdb_httplib_openssl::Client>(proto_host_port);
 	client->set_follow_location(true);
 	client->set_keep_alive(http_params.keep_alive);
@@ -223,66 +246,81 @@ unique_ptr<duckdb_httplib_openssl::Client> HTTPFileSystem::GetClient(const HTTPP
 	client->set_read_timeout(http_params.timeout);
 	client->set_connection_timeout(http_params.timeout);
 	client->set_decompress(false);
-	if (hfs && hfs->http_logger) {
+	if (hfh && hfh->http_logger) {
 		client->set_logger(
-		    hfs->http_logger->GetLogger<duckdb_httplib_openssl::Request, duckdb_httplib_openssl::Response>());
+		    hfh->http_logger->GetLogger<duckdb_httplib_openssl::Request, duckdb_httplib_openssl::Response>());
 	}
 	if (!http_params.bearer_token.empty()) {
 		client->set_bearer_token_auth(http_params.bearer_token.c_str());
 	}
+
+	if (!http_params.http_proxy.empty()) {
+		client->set_proxy(http_params.http_proxy, http_params.http_proxy_port);
+
+		if (!http_params.http_proxy_username.empty()) {
+			client->set_proxy_basic_auth(http_params.http_proxy_username, http_params.http_proxy_password);
+		}
+	}
+
 	return client;
 }
 
 unique_ptr<ResponseWrapper> HTTPFileSystem::PutRequest(FileHandle &handle, string url, HeaderMap header_map,
                                                        char *buffer_in, idx_t buffer_in_len, string params) {
-	auto &hfs = handle.Cast<HTTPFileHandle>();
+	auto &hfh = handle.Cast<HTTPFileHandle>();
 	string path, proto_host_port;
 	ParseUrl(url, path, proto_host_port);
-	auto headers = initialize_http_headers(header_map);
+	auto headers = InitializeHeaders(header_map, hfh.http_params);
 
 	std::function<duckdb_httplib_openssl::Result(void)> request([&]() {
-		auto client = GetClient(hfs.http_params, proto_host_port.c_str(), &hfs);
-		if (hfs.state) {
-			hfs.state->put_count++;
-			hfs.state->total_bytes_sent += buffer_in_len;
+		auto client = GetClient(hfh.http_params, proto_host_port.c_str(), &hfh);
+		if (hfh.state) {
+			hfh.state->put_count++;
+			hfh.state->total_bytes_sent += buffer_in_len;
 		}
 		return client->Put(path.c_str(), *headers, buffer_in, buffer_in_len, "application/octet-stream");
 	});
 
-	return RunRequestWithRetry(request, url, "PUT", hfs.http_params);
+	return RunRequestWithRetry(request, url, "PUT", hfh.http_params);
 }
 
 unique_ptr<ResponseWrapper> HTTPFileSystem::HeadRequest(FileHandle &handle, string url, HeaderMap header_map) {
-	auto &hfs = handle.Cast<HTTPFileHandle>();
+	auto &hfh = handle.Cast<HTTPFileHandle>();
 	string path, proto_host_port;
 	ParseUrl(url, path, proto_host_port);
-	auto headers = initialize_http_headers(header_map);
+	auto headers = InitializeHeaders(header_map, hfh.http_params);
+	auto http_client = hfh.GetClient(nullptr);
 
 	std::function<duckdb_httplib_openssl::Result(void)> request([&]() {
-		if (hfs.state) {
-			hfs.state->head_count++;
+		if (hfh.state) {
+			hfh.state->head_count++;
 		}
-		return hfs.http_client->Head(path.c_str(), *headers);
+		return http_client->Head(path.c_str(), *headers);
 	});
 
+	// Refresh the client on retries
 	std::function<void(void)> on_retry(
-	    [&]() { hfs.http_client = GetClient(hfs.http_params, proto_host_port.c_str(), &hfs); });
+	    [&]() { http_client = GetClient(hfh.http_params, proto_host_port.c_str(), &hfh); });
 
-	return RunRequestWithRetry(request, url, "HEAD", hfs.http_params, on_retry);
+	auto response = RunRequestWithRetry(request, url, "HEAD", hfh.http_params, on_retry);
+	hfh.StoreClient(std::move(http_client));
+	return response;
 }
 
 unique_ptr<ResponseWrapper> HTTPFileSystem::GetRequest(FileHandle &handle, string url, HeaderMap header_map) {
 	auto &hfh = handle.Cast<HTTPFileHandle>();
 	string path, proto_host_port;
 	ParseUrl(url, path, proto_host_port);
-	auto headers = initialize_http_headers(header_map);
+	auto headers = InitializeHeaders(header_map, hfh.http_params);
 
 	D_ASSERT(hfh.cached_file_handle);
 
+	auto http_client = hfh.GetClient(nullptr);
+
 	std::function<duckdb_httplib_openssl::Result(void)> request([&]() {
 		D_ASSERT(hfh.state);
 		hfh.state->get_count++;
-		return hfh.http_client->Get(
+		return http_client->Get(
 		    path.c_str(), *headers,
 		    [&](const duckdb_httplib_openssl::Response &response) {
 			    if (response.status >= 400) {
@@ -322,29 +360,33 @@ unique_ptr<ResponseWrapper> HTTPFileSystem::GetRequest(FileHandle &handle, strin
 	});
 
 	std::function<void(void)> on_retry(
-	    [&]() { hfh.http_client = GetClient(hfh.http_params, proto_host_port.c_str(), &hfh); });
+	    [&]() { http_client = GetClient(hfh.http_params, proto_host_port.c_str(), &hfh); });
 
-	return RunRequestWithRetry(request, url, "GET", hfh.http_params, on_retry);
+	auto response = RunRequestWithRetry(request, url, "GET", hfh.http_params, on_retry);
+	hfh.StoreClient(std::move(http_client));
+	return response;
 }
 
 unique_ptr<ResponseWrapper> HTTPFileSystem::GetRangeRequest(FileHandle &handle, string url, HeaderMap header_map,
                                                             idx_t file_offset, char *buffer_out, idx_t buffer_out_len) {
-	auto &hfs = handle.Cast<HTTPFileHandle>();
+	auto &hfh = handle.Cast<HTTPFileHandle>();
 	string path, proto_host_port;
 	ParseUrl(url, path, proto_host_port);
-	auto headers = initialize_http_headers(header_map);
+	auto headers = InitializeHeaders(header_map, hfh.http_params);
 
 	// send the Range header to read only subset of file
 	string range_expr = "bytes=" + to_string(file_offset) + "-" + to_string(file_offset + buffer_out_len - 1);
 	headers->insert(pair<string, string>("Range", range_expr));
 
+	auto http_client = hfh.GetClient(nullptr);
+
 	idx_t out_offset = 0;
 
 	std::function<duckdb_httplib_openssl::Result(void)> request([&]() {
-		if (hfs.state) {
-			hfs.state->get_count++;
+		if (hfh.state) {
+			hfh.state->get_count++;
 		}
-		return hfs.http_client->Get(
+		return http_client->Get(
 		    path.c_str(), *headers,
 		    [&](const duckdb_httplib_openssl::Response &response) {
 			    if (response.status >= 400) {
@@ -368,8 +410,8 @@ unique_ptr<ResponseWrapper> HTTPFileSystem::GetRangeRequest(FileHandle &handle,
 			    return true;
 		    },
 		    [&](const char *data, size_t data_length) {
-			    if (hfs.state) {
-				    hfs.state->total_bytes_received += data_length;
+			    if (hfh.state) {
+				    hfh.state->total_bytes_received += data_length;
 			    }
 			    if (buffer_out != nullptr) {
 				    if (data_length + out_offset > buffer_out_len) {
@@ -389,9 +431,11 @@ unique_ptr<ResponseWrapper> HTTPFileSystem::GetRangeRequest(FileHandle &handle,
 	});
 
 	std::function<void(void)> on_retry(
-	    [&]() { hfs.http_client = GetClient(hfs.http_params, proto_host_port.c_str(), &hfs); });
+	    [&]() { http_client = GetClient(hfh.http_params, proto_host_port.c_str(), &hfh); });
 
-	return RunRequestWithRetry(request, url, "GET Range", hfs.http_params, on_retry);
+	auto response = RunRequestWithRetry(request, url, "GET Range", hfh.http_params, on_retry);
+	hfh.StoreClient(std::move(http_client));
+	return response;
 }
 
 HTTPFileHandle::HTTPFileHandle(FileSystem &fs, const string &path, FileOpenFlags flags, const HTTPParams &http_params)
@@ -403,7 +447,9 @@ unique_ptr<HTTPFileHandle> HTTPFileSystem::CreateHandle(const string &path, File
                                                         optional_ptr<FileOpener> opener) {
 	D_ASSERT(flags.Compression() == FileCompressionType::UNCOMPRESSED);
 
-	auto params = HTTPParams::ReadFrom(opener);
+	FileOpenerInfo info;
+	info.file_path = path;
+	auto params = HTTPParams::ReadFrom(opener, info);
 
 	auto secret_manager = FileOpener::TryGetSecretManager(opener);
 	auto transaction = FileOpener::TryGetCatalogTransaction(opener);
@@ -416,7 +462,14 @@ unique_ptr<HTTPFileHandle> HTTPFileSystem::CreateHandle(const string &path, File
 		}
 	}
 
-	return duckdb::make_uniq<HTTPFileHandle>(*this, path, flags, params);
+	auto result = duckdb::make_uniq<HTTPFileHandle>(*this, path, flags, params);
+
+	auto client_context = FileOpener::TryGetClientContext(opener);
+	if (client_context && ClientConfig::GetConfig(*client_context).enable_http_logging) {
+		result->http_logger = client_context->client_data->http_logger.get();
+	}
+
+	return result;
 }
 
 unique_ptr<FileHandle> HTTPFileSystem::OpenFile(const string &path, FileOpenFlags flags,
@@ -591,26 +644,23 @@ static optional_ptr<HTTPMetadataCache> TryGetMetadataCache(optional_ptr<FileOpen
 	if (use_shared_cache) {
 		return httpfs.GetGlobalCache();
 	} else if (client_context) {
-		auto lookup = client_context->registered_state.find("http_cache");
-		if (lookup == client_context->registered_state.end()) {
-			auto cache = make_shared_ptr<HTTPMetadataCache>(true, true);
-			client_context->registered_state["http_cache"] = cache;
-			return cache.get();
-		} else {
-			return (HTTPMetadataCache *)lookup->second.get();
-		}
+		return client_context->registered_state->GetOrCreate<HTTPMetadataCache>("http_cache", true, true).get();
 	}
 	return nullptr;
 }
 
 void HTTPFileHandle::Initialize(optional_ptr<FileOpener> opener) {
-	InitializeClient(FileOpener::TryGetClientContext(opener));
 	auto &hfs = file_system.Cast<HTTPFileSystem>();
 	state = HTTPState::TryGetState(opener);
 	if (!state) {
 		state = make_shared_ptr<HTTPState>();
 	}
 
+	auto client_context = FileOpener::TryGetClientContext(opener);
+	if (client_context && ClientConfig::GetConfig(*client_context).enable_http_logging) {
+		http_logger = client_context->client_data->http_logger.get();
+	}
+
 	auto current_cache = TryGetMetadataCache(opener, hfs);
 
 	bool should_write_cache = false;
@@ -737,15 +787,32 @@ void HTTPFileHandle::Initialize(optional_ptr<FileOpener> opener) {
 	}
 }
 
-void HTTPFileHandle::InitializeClient(optional_ptr<ClientContext> context) {
+unique_ptr<duckdb_httplib_openssl::Client> HTTPFileHandle::GetClient(optional_ptr<ClientContext> context) {
+	// Try to fetch a cached client
+	auto cached_client = client_cache.GetClient();
+	if (cached_client) {
+		return cached_client;
+	}
+
+	// Create a new client
+	return CreateClient(context);
+}
+
+unique_ptr<duckdb_httplib_openssl::Client> HTTPFileHandle::CreateClient(optional_ptr<ClientContext> context) {
+	// Create a new client
 	string path_out, proto_host_port;
 	HTTPFileSystem::ParseUrl(path, path_out, proto_host_port);
-	http_client = HTTPFileSystem::GetClient(this->http_params, proto_host_port.c_str(), this);
+	auto http_client = HTTPFileSystem::GetClient(this->http_params, proto_host_port.c_str(), this);
 	if (context && ClientConfig::GetConfig(*context).enable_http_logging) {
 		http_logger = context->client_data->http_logger.get();
 		http_client->set_logger(
 		    http_logger->GetLogger<duckdb_httplib_openssl::Request, duckdb_httplib_openssl::Response>());
 	}
+	return http_client;
+}
+
+void HTTPFileHandle::StoreClient(unique_ptr<duckdb_httplib_openssl::Client> client) {
+	client_cache.StoreClient(std::move(client));
 }
 
 ResponseWrapper::ResponseWrapper(duckdb_httplib_openssl::Response &res, string &original_url) {
