@@ -3,39 +3,44 @@ import json
 import re
 import argparse
 
+from typing import Dict, Optional, Tuple, List
+
 parser = argparse.ArgumentParser(description='Generate serialization code')
 parser.add_argument('--source', type=str, help='Source directory')
 parser.add_argument('--target', type=str, help='Target directory')
 
 args = parser.parse_args()
 
-if args.source is None:
-    targets = [
-        {'source': 'src/include/duckdb/storage/serialization', 'target': 'src/storage/serialization'},
-        {'source': 'extension/parquet/include/', 'target': 'extension/parquet'},
-        {'source': 'extension/json/include/', 'target': 'extension/json'},
-    ]
-else:
-    targets = [
-        {'source': args.source, 'target': args.target},
-    ]
 
+def get_file_list():
+    if args.source is None:
+        targets = [
+            {'source': 'src/include/duckdb/storage/serialization', 'target': 'src/storage/serialization'},
+            {'source': 'extension/parquet/include/', 'target': 'extension/parquet'},
+            {'source': 'extension/json/include/', 'target': 'extension/json'},
+        ]
+    else:
+        targets = [
+            {'source': args.source, 'target': args.target},
+        ]
+
+    file_list = []
+    for target in targets:
+        source_base = os.path.sep.join(target['source'].split('/'))
+        target_base = os.path.sep.join(target['target'].split('/'))
+        for fname in os.listdir(source_base):
+            if '.json' not in fname:
+                continue
+            if '_enums.json' in fname:
+                continue
+            file_list.append(
+                {
+                    'source': os.path.join(source_base, fname),
+                    'target': os.path.join(target_base, 'serialize_' + fname.replace('.json', '.cpp')),
+                }
+            )
+    return file_list
 
-file_list = []
-for target in targets:
-    source_base = os.path.sep.join(target['source'].split('/'))
-    target_base = os.path.sep.join(target['target'].split('/'))
-    for fname in os.listdir(source_base):
-        if '.json' not in fname:
-            continue
-        if '_enums.json' in fname:
-            continue
-        file_list.append(
-            {
-                'source': os.path.join(source_base, fname),
-                'target': os.path.join(target_base, 'serialize_' + fname.replace('.json', '.cpp')),
-            }
-        )
 
 scripts_dir = os.path.dirname(os.path.abspath(__file__))
 version_map_path = scripts_dir + '/../src/storage/version_map.json'
@@ -90,66 +95,68 @@ def lookup_serialization_version(version: str):
     return versions[version]
 
 
-include_base = '#include "${FILENAME}"\n'
+INCLUDE_FORMAT = '#include "{filename}"\n'
 
-header = '''//===----------------------------------------------------------------------===//
+HEADER = '''//===----------------------------------------------------------------------===//
 // This file is automatically generated by scripts/generate_serialization.py
 // Do not edit this file manually, your changes will be overwritten
 //===----------------------------------------------------------------------===//
 
-${INCLUDE_LIST}
-namespace duckdb {
+{include_list}
+namespace duckdb {{
 '''
 
-footer = '''
+FOOTER = '''
 } // namespace duckdb
 '''
 
-templated_base = '''
-template <typename ${TEMPLATE_NAME}>'''
+TEMPLATED_BASE_FORMAT = '''
+template <typename {template_name}>'''
 
-serialize_base = '''
-void ${CLASS_NAME}::Serialize(Serializer &serializer) const {
-${MEMBERS}}
+SERIALIZE_BASE_FORMAT = '''
+void {class_name}::Serialize(Serializer &serializer) const {{
+{members}}}
 '''
 
-serialize_element = '\tserializer.WriteProperty<${PROPERTY_TYPE}>(${PROPERTY_ID}, "${PROPERTY_KEY}", ${PROPERTY_NAME}${PROPERTY_DEFAULT});\n'
+SERIALIZE_ELEMENT_FORMAT = (
+    '\tserializer.WriteProperty<{property_type}>({property_id}, "{property_key}", {property_name}{property_default});\n'
+)
 
-base_serialize = '\t${BASE_CLASS_NAME}::Serialize(serializer);\n'
+BASE_SERIALIZE_FORMAT = '\t{base_class_name}::Serialize(serializer);\n'
 
-pointer_return = '${POINTER}<${CLASS_NAME}>'
+POINTER_RETURN_FORMAT = '{pointer}<{class_name}>'
 
-deserialize_base = '''
-${DESERIALIZE_RETURN} ${CLASS_NAME}::Deserialize(Deserializer &deserializer) {
-${MEMBERS}
-}
+DESERIALIZE_BASE_FORMAT = '''
+{deserialize_return} {class_name}::Deserialize(Deserializer &deserializer) {{
+{members}
+}}
 '''
 
-switch_code = '''\tswitch (${SWITCH_VARIABLE}) {
-${CASE_STATEMENTS}\tdefault:
-\t\tthrow SerializationException("Unsupported type for deserialization of ${BASE_CLASS}!");
-\t}
+SWITCH_CODE_FORMAT = '''\tswitch ({switch_variable}) {{
+{case_statements}\tdefault:
+\t\tthrow SerializationException("Unsupported type for deserialization of {base_class}!");
+\t}}
 '''
 
-set_deserialize_parameter = '\tdeserializer.Set<${PROPERTY_TYPE}>(${PROPERTY_NAME});\n'
-unset_deserialize_parameter = '\tdeserializer.Unset<${PROPERTY_TYPE}>();\n'
-get_deserialize_parameter = 'deserializer.Get<${PROPERTY_TYPE}>()'
+SET_DESERIALIZE_PARAMETER_FORMAT = '\tdeserializer.Set<{property_type}>({property_name});\n'
+UNSET_DESERIALIZE_PARAMETER_FORMAT = '\tdeserializer.Unset<{property_type}>();\n'
+GET_DESERIALIZE_PARAMETER_FORMAT = 'deserializer.Get<{property_type}>()'
 
-switch_header = '\tcase ${ENUM_TYPE}::${ENUM_VALUE}:\n'
+SWITCH_HEADER_FORMAT = '\tcase {enum_type}::{enum_value}:\n'
 
-switch_statement = (
-    switch_header
-    + '''\t\tresult = ${CLASS_DESERIALIZE}::Deserialize(deserializer);
+SWITCH_STATEMENT_FORMAT = (
+    SWITCH_HEADER_FORMAT
+    + '''\t\tresult = {class_deserialize}::Deserialize(deserializer);
 \t\tbreak;
 '''
 )
 
-deserialize_element = '\tauto ${PROPERTY_NAME} = deserializer.ReadProperty<${PROPERTY_TYPE}>(${PROPERTY_ID}, "${PROPERTY_KEY}"${PROPERTY_DEFAULT});\n'
-deserialize_element_base = '\tauto ${PROPERTY_NAME} = deserializer.ReadProperty<unique_ptr<${BASE_PROPERTY}>>(${PROPERTY_ID}, "${PROPERTY_KEY}"${PROPERTY_DEFAULT});\n'
-deserialize_element_class = '\tdeserializer.ReadProperty<${PROPERTY_TYPE}>(${PROPERTY_ID}, "${PROPERTY_KEY}", result${ASSIGNMENT}${PROPERTY_NAME}${PROPERTY_DEFAULT});\n'
-deserialize_element_class_base = '\tauto ${PROPERTY_NAME} = deserializer.ReadProperty<unique_ptr<${BASE_PROPERTY}>>(${PROPERTY_ID}, "${PROPERTY_KEY}"${PROPERTY_DEFAULT});\n\tresult${ASSIGNMENT}${PROPERTY_NAME} = unique_ptr_cast<${BASE_PROPERTY}, ${DERIVED_PROPERTY}>(std::move(${PROPERTY_NAME}));\n'
+DESERIALIZE_ELEMENT_FORMAT = '\tauto {property_name} = deserializer.ReadProperty<{property_type}>({property_id}, "{property_key}"{property_default});\n'
+DESERIALIZE_ELEMENT_BASE_FORMAT = '\tauto {property_name} = deserializer.ReadProperty<unique_ptr<{base_property}>>({property_id}, "{property_key}"{property_default});\n'
+DESERIALIZE_ELEMENT_CLASS_FORMAT = '\tdeserializer.ReadProperty<{property_type}>({property_id}, "{property_key}", result{assignment}{property_name}{property_default});\n'
+DESERIALIZE_ELEMENT_CLASS_BASE_FORMAT = '\tauto {property_name} = deserializer.ReadProperty<unique_ptr<{base_property}>>({property_id}, "{property_key}"{property_default});\n\tresult{assignment}{property_name} = unique_ptr_cast<{base_property}, {derived_property}>(std::move({property_name}));\n'
 
-move_list = [
+MOVE_LIST = [
     'string',
     'ParsedExpression*',
     'CommonTableExpressionMap',
@@ -159,7 +166,7 @@ move_list = [
     'BoundLimitNode',
 ]
 
-reference_list = ['ClientContext', 'bound_parameter_map_t']
+REFERENCE_LIST = ['ClientContext', 'bound_parameter_map_t']
 
 
 def is_container(type):
@@ -188,7 +195,7 @@ def is_zeroable(type):
 
 
 def requires_move(type):
-    return is_container(type) or is_pointer(type) or type in move_list
+    return is_container(type) or is_pointer(type) or type in MOVE_LIST
 
 
 def replace_pointer(type):
@@ -212,18 +219,18 @@ def get_serialize_element(
 ):
     assignment = '.' if pointer_type == 'none' else '->'
     default_argument = '' if default_value is None else f', {get_default_argument(default_value)}'
-    template = serialize_element
+    template = SERIALIZE_ELEMENT_FORMAT
     if is_deleted:
-        template = "\t/* [Deleted] (${PROPERTY_TYPE}) \"${PROPERTY_NAME}\" */\n"
+        template = "\t/* [Deleted] ({property_type}) \"{property_name}\" */\n"
     elif has_default:
         template = template.replace('WriteProperty', 'WritePropertyWithDefault')
-    serialization_code = (
-        template.replace('${PROPERTY_NAME}', property_name)
-        .replace('${PROPERTY_TYPE}', property_type)
-        .replace('${PROPERTY_ID}', str(property_id))
-        .replace('${PROPERTY_KEY}', property_key)
-        .replace('${PROPERTY_DEFAULT}', default_argument)
-        .replace('${ASSIGNMENT}', assignment)
+    serialization_code = template.format(
+        property_name=property_name,
+        property_type=property_type,
+        property_id=str(property_id),
+        property_key=property_key,
+        property_default=default_argument,
+        assignment=assignment,
     )
 
     storage_version = lookup_serialization_version(version)
@@ -252,27 +259,29 @@ def get_deserialize_element_template(
     assignment = '.' if pointer_type == 'none' else '->'
     default_argument = '' if default_value is None else f', {get_default_argument(default_value)}'
     if is_deleted:
-        template = template.replace(', result${ASSIGNMENT}${PROPERTY_NAME}', '').replace(
+        template = template.replace(', result{assignment}{property_name}', '').replace(
             'ReadProperty', 'ReadDeletedProperty'
         )
-    elif has_default:
+    elif has_default and default_value is None:
         template = template.replace('ReadProperty', 'ReadPropertyWithDefault')
-    return (
-        template.replace('${PROPERTY_NAME}', property_name)
-        .replace('${PROPERTY_KEY}', property_key)
-        .replace('${PROPERTY_ID}', str(property_id))
-        .replace('${PROPERTY_DEFAULT}', default_argument)
-        .replace('${PROPERTY_TYPE}', property_type)
-        .replace('${ASSIGNMENT}', assignment)
+    elif has_default and default_value is not None:
+        template = template.replace('ReadProperty', 'ReadPropertyWithExplicitDefault')
+    return template.format(
+        property_name=property_name,
+        property_key=property_key,
+        property_id=str(property_id),
+        property_default=default_argument,
+        property_type=property_type,
+        assignment=assignment,
     )
 
 
 def get_deserialize_element(
     property_name, property_key, property_id, property_type, has_default, default_value, is_deleted, base, pointer_type
 ):
-    template = deserialize_element
+    template = DESERIALIZE_ELEMENT_FORMAT
     if base:
-        template = deserialize_element_base.replace('${BASE_PROPERTY}', base.replace('*', ''))
+        template = DESERIALIZE_ELEMENT_BASE_FORMAT.replace('{base_property}', base.replace('*', ''))
 
     return get_deserialize_element_template(
         template,
@@ -298,7 +307,7 @@ def get_deserialize_assignment(property_name, property_type, pointer_type):
 def get_return_value(pointer_type, class_name):
     if pointer_type == 'none':
         return class_name
-    return pointer_return.replace('${POINTER}', pointer_type).replace('${CLASS_NAME}', class_name)
+    return POINTER_RETURN_FORMAT.format(pointer=pointer_type, class_name=class_name)
 
 
 def generate_constructor(pointer_type, class_name, constructor_parameters):
@@ -414,10 +423,10 @@ class SerializableClass:
         self.set_parameters = []
         self.pointer_type = 'unique_ptr'
         self.constructor = None
-        self.members = None
+        self.members: Optional[List[MemberVariable]] = None
         self.custom_implementation = False
         self.custom_switch_code = None
-        self.children = {}
+        self.children: Dict[str, SerializableClass] = {}
         self.return_type = self.name
         self.return_class = self.name
         if self.is_base_class:
@@ -445,6 +454,7 @@ class SerializableClass:
             self.set_parameter_names = entry['set_parameters']
             for set_parameter_name in self.set_parameter_names:
                 found = False
+                assert self.members is not None
                 for member in self.members:
                     if member.name == set_parameter_name:
                         self.set_parameters.append(member)
@@ -501,8 +511,8 @@ def generate_base_class_code(base_class):
 
     # set parameters
     for entry in base_class.set_parameters:
-        base_class_deserialize += set_deserialize_parameter.replace('${PROPERTY_TYPE}', entry.type).replace(
-            '${PROPERTY_NAME}', entry.name
+        base_class_deserialize += SET_DESERIALIZE_PARAMETER_FORMAT.format(
+            property_type=entry.type, property_name=entry.name
         )
 
     base_class_deserialize += f'\t{base_class.pointer_type}<{base_class.name}> result;\n'
@@ -511,20 +521,16 @@ def generate_base_class_code(base_class):
         enum_value = expr[0]
         child_data = expr[1]
         if child_data.custom_switch_code is not None:
-            switch_cases += (
-                switch_header.replace('${ENUM_TYPE}', enum_type)
-                .replace('${ENUM_VALUE}', enum_value)
-                .replace('${CLASS_DESERIALIZE}', child_data.name)
+            switch_cases += SWITCH_HEADER_FORMAT.format(
+                enum_type=enum_type, enum_value=enum_value, class_deserialize=child_data.name
             )
             switch_cases += '\n'.join(
                 ['\t\t' + x for x in child_data.custom_switch_code.replace('\\n', '\n').split('\n')]
             )
             switch_cases += '\n'
             continue
-        switch_cases += (
-            switch_statement.replace('${ENUM_TYPE}', enum_type)
-            .replace('${ENUM_VALUE}', enum_value)
-            .replace('${CLASS_DESERIALIZE}', child_data.name)
+        switch_cases += SWITCH_STATEMENT_FORMAT.format(
+            enum_type=enum_type, enum_value=enum_value, class_deserialize=child_data.name
         )
 
     assign_entries = []
@@ -540,20 +546,18 @@ def generate_base_class_code(base_class):
         assign_entries.append(entry)
 
     # class switch statement
-    base_class_deserialize += (
-        switch_code.replace('${SWITCH_VARIABLE}', base_class.enum_value)
-        .replace('${CASE_STATEMENTS}', switch_cases)
-        .replace('${BASE_CLASS}', base_class.name)
+    base_class_deserialize += SWITCH_CODE_FORMAT.format(
+        switch_variable=base_class.enum_value, case_statements=switch_cases, base_class=base_class.name
     )
 
     deserialize_return = get_return_value(base_class.pointer_type, base_class.return_type)
 
     for entry in base_class.set_parameters:
-        base_class_deserialize += unset_deserialize_parameter.replace('${PROPERTY_TYPE}', entry.type)
+        base_class_deserialize += UNSET_DESERIALIZE_PARAMETER_FORMAT.format(property_type=entry.type)
 
     for entry in assign_entries:
         move = False
-        if entry.type in move_list or is_container(entry.type) or is_pointer(entry.type):
+        if entry.type in MOVE_LIST or is_container(entry.type) or is_pointer(entry.type):
             move = True
         if move:
             base_class_deserialize += (
@@ -565,14 +569,12 @@ def generate_base_class_code(base_class):
     base_class_generation = ''
     serialization = ''
     if base_class.base is not None:
-        serialization += base_serialize.replace('${BASE_CLASS_NAME}', base_class.base)
-    base_class_generation += serialize_base.replace('${CLASS_NAME}', base_class.name).replace(
-        '${MEMBERS}', serialization + base_class_serialize
+        serialization += BASE_SERIALIZE_FORMAT.format(base_class_name=base_class.base)
+    base_class_generation += SERIALIZE_BASE_FORMAT.format(
+        class_name=base_class.name, members=serialization + base_class_serialize
     )
-    base_class_generation += (
-        deserialize_base.replace('${DESERIALIZE_RETURN}', deserialize_return)
-        .replace('${CLASS_NAME}', base_class.name)
-        .replace('${MEMBERS}', base_class_deserialize)
+    base_class_generation += DESERIALIZE_BASE_FORMAT.format(
+        deserialize_return=deserialize_return, class_name=base_class.name, members=base_class_deserialize
     )
     return base_class_generation
 
@@ -616,23 +618,23 @@ def generate_class_code(class_entry):
                 if len(constructor_parameters) > 0:
                     constructor_parameters += ", "
                 param_type = constructor_entry.replace('$', '')
-                if param_type in reference_list:
+                if param_type in REFERENCE_LIST:
                     param_type += ' &'
-                constructor_parameters += get_deserialize_parameter.replace('${PROPERTY_TYPE}', param_type)
+                constructor_parameters += GET_DESERIALIZE_PARAMETER_FORMAT.format(property_type=param_type)
                 found = True
             if class_entry.base_object is not None:
                 for entry in class_entry.base_object.set_parameters:
                     if entry.name == constructor_entry:
                         if len(constructor_parameters) > 0:
                             constructor_parameters += ", "
-                        constructor_parameters += get_deserialize_parameter.replace('${PROPERTY_TYPE}', entry.type)
+                        constructor_parameters += GET_DESERIALIZE_PARAMETER_FORMAT.format(property_type=entry.type)
                         found = True
                         break
             if not found:
                 raise Exception(f"Constructor member \"{constructor_entry}\" was not found in members list")
 
     if class_entry.base is not None:
-        class_serialize += base_serialize.replace('${BASE_CLASS_NAME}', class_entry.base)
+        class_serialize += BASE_SERIALIZE_FORMAT.format(base_class_name=class_entry.base)
     for entry_idx in range(last_constructor_index + 1):
         entry = class_entry.members[entry_idx]
         type_name = replace_pointer(entry.type)
@@ -658,12 +660,12 @@ def generate_class_code(class_entry):
         property_id = entry.id
         property_key = entry.name
         write_property_name = entry.serialize_property
-        deserialize_template_str = deserialize_element_class
+        deserialize_template_str = DESERIALIZE_ELEMENT_CLASS_FORMAT
         default_value = entry.default
         if entry.base:
-            deserialize_template_str = deserialize_element_class_base.replace(
-                '${BASE_PROPERTY}', entry.base.replace('*', '')
-            ).replace('${DERIVED_PROPERTY}', entry.type.replace('*', ''))
+            deserialize_template_str = DESERIALIZE_ELEMENT_CLASS_BASE_FORMAT.replace(
+                '{base_property}', entry.base.replace('*', '')
+            ).replace('{derived_property}', entry.type.replace('*', ''))
         type_name = replace_pointer(entry.type)
 
         class_serialize += get_serialize_element(
@@ -694,13 +696,13 @@ def generate_class_code(class_entry):
                 entry.deserialize_property, entry.type, class_entry.pointer_type
             )
         if entry.name in class_entry.set_parameter_names:
-            class_deserialize += set_deserialize_parameter.replace('${PROPERTY_TYPE}', entry.type).replace(
-                '${PROPERTY_NAME}', entry.name
+            class_deserialize += SET_DESERIALIZE_PARAMETER_FORMAT.format(
+                property_type=entry.type, property_name=entry.name
             )
 
     for entry in class_entry.set_parameters:
-        class_deserialize += unset_deserialize_parameter.replace('${PROPERTY_TYPE}', entry.type).replace(
-            '${PROPERTY_NAME}', entry.name
+        class_deserialize += UNSET_DESERIALIZE_PARAMETER_FORMAT.format(
+            property_type=entry.type, property_name=entry.name
         )
     class_deserialize += generate_return(class_entry)
     deserialize_return = get_return_value(class_entry.pointer_type, class_entry.return_type)
@@ -712,21 +714,21 @@ def generate_class_code(class_entry):
     # Check if is a templated class
     is_templated = pattern.search(class_entry.name)
     if is_templated:
-        templated_type = templated_base.replace('${TEMPLATE_NAME}', is_templated.group()[1:-1])
+        templated_type = TEMPLATED_BASE_FORMAT.format(template_name=is_templated.group()[1:-1])
 
-    class_generation += templated_type + serialize_base.replace('${CLASS_NAME}', class_entry.name).replace(
-        '${MEMBERS}', class_serialize
+    class_generation += templated_type + SERIALIZE_BASE_FORMAT.format(
+        class_name=class_entry.name, members=class_serialize
     )
 
-    class_generation += templated_type + (
-        deserialize_base.replace('${DESERIALIZE_RETURN}', deserialize_return)
-        .replace('${CLASS_NAME}', class_entry.name)
-        .replace('${MEMBERS}', class_deserialize)
+    class_generation += templated_type + DESERIALIZE_BASE_FORMAT.format(
+        deserialize_return=deserialize_return,
+        class_name=class_entry.name,
+        members=class_deserialize,
     )
     return class_generation
 
 
-def check_children_for_duplicate_members(node, parents: list, seen_names: set, seen_ids: set):
+def check_children_for_duplicate_members(node: SerializableClass, parents: list, seen_names: set, seen_ids: set):
     # Check for duplicate names
     if node.members is not None:
         for member in node.members:
@@ -747,6 +749,8 @@ def check_children_for_duplicate_members(node, parents: list, seen_names: set, s
         check_children_for_duplicate_members(child, parents + [node], seen_names.copy(), seen_ids.copy())
 
 
+file_list = get_file_list()
+
 for entry in file_list:
     source_path = entry['source']
     target_path = entry['target']
@@ -761,9 +765,9 @@ for entry in file_list:
         'duckdb/common/serializer/serializer.hpp',
         'duckdb/common/serializer/deserializer.hpp',
     ]
-    base_classes = []
-    classes = []
-    base_class_data = {}
+    base_classes: List[SerializableClass] = []
+    classes: List[SerializableClass] = []
+    base_class_data: Dict[str, SerializableClass] = {}
 
     for entry in json_data:
         if 'includes' in entry:
@@ -795,9 +799,9 @@ for entry in file_list:
             check_children_for_duplicate_members(base_class, [], set(), set())
 
     with open(target_path, 'w+') as f:
-        f.write(
-            header.replace('${INCLUDE_LIST}', ''.join([include_base.replace('${FILENAME}', x) for x in include_list]))
-        )
+        include_list = ''.join([INCLUDE_FORMAT.format(filename=x) for x in include_list])
+        header = HEADER.format(include_list=include_list)
+        f.write(header)
 
         # generate the base class serialization
         for base_class in base_classes:
@@ -812,4 +816,4 @@ for entry in file_list:
                 continue
             f.write(class_generation)
 
-        f.write(footer)
+        f.write(FOOTER)
