@@ -95,8 +95,8 @@ void CSVSniffer::SetDateFormat(CSVStateMachine &candidate, const string &format_
 	candidate.dialect_options.date_format[sql_type].Set(strpformat, false);
 }
 
-bool CSVSniffer::CanYouCastIt(const string_t value, const LogicalType &type, const DialectOptions &dialect_options,
-                              const bool is_null, const char decimal_separator) {
+bool CSVSniffer::CanYouCastIt(ClientContext &context, const string_t value, const LogicalType &type,
+                              const DialectOptions &dialect_options, const bool is_null, const char decimal_separator) {
 	if (is_null) {
 		return true;
 	}
@@ -137,11 +137,11 @@ bool CSVSniffer::CanYouCastIt(const string_t value, const LogicalType &type, con
 	}
 	case LogicalTypeId::DOUBLE: {
 		double dummy_value;
-		return TryDoubleCast<double>(value_ptr, value_size, dummy_value, true, options.decimal_separator[0]);
+		return TryDoubleCast<double>(value_ptr, value_size, dummy_value, true, decimal_separator);
 	}
 	case LogicalTypeId::FLOAT: {
 		float dummy_value;
-		return TryDoubleCast<float>(value_ptr, value_size, dummy_value, true, options.decimal_separator[0]);
+		return TryDoubleCast<float>(value_ptr, value_size, dummy_value, true, decimal_separator);
 	}
 	case LogicalTypeId::DATE: {
 		if (!dialect_options.date_format.find(LogicalTypeId::DATE)->second.GetValue().Empty()) {
@@ -150,12 +150,11 @@ bool CSVSniffer::CanYouCastIt(const string_t value, const LogicalType &type, con
 			return dialect_options.date_format.find(LogicalTypeId::DATE)
 			    ->second.GetValue()
 			    .TryParseDate(value, result, error_message);
-		} else {
-			idx_t pos;
-			bool special;
-			date_t dummy_value;
-			return Date::TryConvertDate(value_ptr, value_size, pos, dummy_value, special, true);
 		}
+		idx_t pos;
+		bool special;
+		date_t dummy_value;
+		return Date::TryConvertDate(value_ptr, value_size, pos, dummy_value, special, true);
 	}
 	case LogicalTypeId::TIMESTAMP: {
 		timestamp_t dummy_value;
@@ -164,9 +163,8 @@ bool CSVSniffer::CanYouCastIt(const string_t value, const LogicalType &type, con
 			return dialect_options.date_format.find(LogicalTypeId::TIMESTAMP)
 			    ->second.GetValue()
 			    .TryParseTimestamp(value, dummy_value, error_message);
-		} else {
-			return Timestamp::TryConvertTimestamp(value_ptr, value_size, dummy_value) == TimestampCastResult::SUCCESS;
 		}
+		return Timestamp::TryConvertTimestamp(value_ptr, value_size, dummy_value) == TimestampCastResult::SUCCESS;
 	}
 	case LogicalTypeId::TIME: {
 		idx_t pos;
@@ -229,9 +227,8 @@ bool CSVSniffer::CanYouCastIt(const string_t value, const LogicalType &type, con
 				throw InternalException("Invalid Physical Type for Decimal Value. Physical Type: " +
 				                        TypeIdToString(type.InternalType()));
 			}
-		} else {
-			throw InvalidInputException("Decimals can only have ',' and '.' as decimal separators");
 		}
+		throw InvalidInputException("Decimals can only have ',' and '.' as decimal separators");
 	}
 	case LogicalTypeId::VARCHAR:
 		return true;
@@ -240,7 +237,7 @@ bool CSVSniffer::CanYouCastIt(const string_t value, const LogicalType &type, con
 		Value new_value;
 		string error_message;
 		Value str_value(value);
-		return str_value.TryCastAs(buffer_manager->context, type, new_value, &error_message, true);
+		return str_value.TryCastAs(context, type, new_value, &error_message, true);
 	}
 	}
 }
@@ -274,8 +271,15 @@ void CSVSniffer::InitializeDateAndTimeStampDetection(CSVStateMachine &candidate,
 	SetDateFormat(candidate, format_candidate.format.back(), sql_type.id());
 }
 
+bool ValidSeparator(const string &separator) {
+	// We use https://en.wikipedia.org/wiki/List_of_date_formats_by_country as reference
+	return separator == "-" || separator == "." || separator == "/" || separator == " ";
+}
 void CSVSniffer::DetectDateAndTimeStampFormats(CSVStateMachine &candidate, const LogicalType &sql_type,
-                                               const string &separator, string_t &dummy_val) {
+                                               const string &separator, const string_t &dummy_val) {
+	if (!ValidSeparator(separator)) {
+		return;
+	}
 	// If it is the first time running date/timestamp detection we must initialize the format variables
 	InitializeDateAndTimeStampDetection(candidate, separator, sql_type);
 	// generate date format candidates the first time through
@@ -289,7 +293,7 @@ void CSVSniffer::DetectDateAndTimeStampFormats(CSVStateMachine &candidate, const
 	while (!type_format_candidates.empty()) {
 		//	avoid using exceptions for flow control...
 		auto &current_format = candidate.dialect_options.date_format[sql_type.id()].GetValue();
-		if (current_format.Parse(dummy_val, result)) {
+		if (current_format.Parse(dummy_val, result, true)) {
 			format_candidates[sql_type.id()].had_match = true;
 			break;
 		}
@@ -317,6 +321,76 @@ void CSVSniffer::DetectDateAndTimeStampFormats(CSVStateMachine &candidate, const
 	}
 }
 
+void CSVSniffer::SniffTypes(DataChunk &data_chunk, CSVStateMachine &state_machine,
+                            unordered_map<idx_t, vector<LogicalType>> &info_sql_types_candidates,
+                            idx_t start_idx_detection) {
+	const idx_t chunk_size = data_chunk.size();
+	HasType has_type;
+	for (idx_t col_idx = 0; col_idx < data_chunk.ColumnCount(); col_idx++) {
+		auto &cur_vector = data_chunk.data[col_idx];
+		D_ASSERT(cur_vector.GetVectorType() == VectorType::FLAT_VECTOR);
+		D_ASSERT(cur_vector.GetType() == LogicalType::VARCHAR);
+		auto vector_data = FlatVector::GetData<string_t>(cur_vector);
+		auto null_mask = FlatVector::Validity(cur_vector);
+		auto &col_type_candidates = info_sql_types_candidates[col_idx];
+		for (idx_t row_idx = start_idx_detection; row_idx < chunk_size; row_idx++) {
+			// col_type_candidates can't be empty since anything in a CSV file should at least be a string
+			// and we validate utf-8 compatibility when creating the type
+			D_ASSERT(!col_type_candidates.empty());
+			auto cur_top_candidate = col_type_candidates.back();
+			// try cast from string to sql_type
+			while (col_type_candidates.size() > 1) {
+				const auto &sql_type = col_type_candidates.back();
+				// try formatting for date types if the user did not specify one and it starts with numeric
+				// values.
+				string separator;
+				// If Value is not Null, Has a numeric date format, and the current investigated candidate is
+				// either a timestamp or a date
+				if (null_mask.RowIsValid(row_idx) && StartsWithNumericDate(separator, vector_data[row_idx]) &&
+				    ((col_type_candidates.back().id() == LogicalTypeId::TIMESTAMP && !has_type.timestamp) ||
+				     (col_type_candidates.back().id() == LogicalTypeId::DATE && !has_type.date))) {
+					DetectDateAndTimeStampFormats(state_machine, sql_type, separator, vector_data[row_idx]);
+				}
+				// try cast from string to sql_type
+				if (sql_type == LogicalType::VARCHAR) {
+					// Nothing to convert it to
+					continue;
+				}
+				if (CanYouCastIt(buffer_manager->context, vector_data[row_idx], sql_type, state_machine.dialect_options,
+				                 !null_mask.RowIsValid(row_idx), state_machine.options.decimal_separator[0])) {
+					break;
+				}
+
+				if (row_idx != start_idx_detection && cur_top_candidate == LogicalType::BOOLEAN) {
+					// If we thought this was a boolean value (i.e., T,F, True, False) and it is not, we
+					// immediately pop to varchar.
+					while (col_type_candidates.back() != LogicalType::VARCHAR) {
+						col_type_candidates.pop_back();
+					}
+					break;
+				}
+				col_type_candidates.pop_back();
+			}
+		}
+		if (col_type_candidates.back().id() == LogicalTypeId::DATE) {
+			has_type.date = true;
+		}
+		if (col_type_candidates.back().id() == LogicalTypeId::TIMESTAMP) {
+			has_type.timestamp = true;
+		}
+	}
+}
+
+// If we have a predefined date/timestamp format we set it
+void CSVSniffer::SetUserDefinedDateTimeFormat(CSVStateMachine &candidate) {
+	const vector<LogicalTypeId> data_time_formats {LogicalTypeId::DATE, LogicalTypeId::TIMESTAMP};
+	for (auto &date_time_format : data_time_formats) {
+		auto &user_option = options.dialect_options.date_format.at(date_time_format);
+		if (user_option.IsSetByUser()) {
+			SetDateFormat(candidate, user_option.GetValue().format_specifier, date_time_format);
+		}
+	}
+}
 void CSVSniffer::DetectTypes() {
 	idx_t min_varchar_cols = max_columns_found + 1;
 	idx_t min_errors = NumericLimits<idx_t>::Maximum();
@@ -336,72 +410,36 @@ void CSVSniffer::DetectTypes() {
 
 		// Reset candidate for parsing
 		auto candidate = candidate_cc->UpgradeToStringValueScanner();
-
+		SetUserDefinedDateTimeFormat(*candidate->state_machine);
 		// Parse chunk and read csv with info candidate
 		auto &data_chunk = candidate->ParseChunk().ToChunk();
-		idx_t row_idx = 0;
+		if (!candidate->error_handler->errors.empty()) {
+			bool break_loop = false;
+			for (auto &errors : candidate->error_handler->errors) {
+				for (auto &error : errors.second) {
+					if (error.type != CSVErrorType::MAXIMUM_LINE_SIZE) {
+						break_loop = true;
+						break;
+					}
+				}
+			}
+			if (break_loop) {
+				continue;
+			}
+		}
+		idx_t start_idx_detection = 0;
 		idx_t chunk_size = data_chunk.size();
 		if (chunk_size > 1 &&
 		    (!options.dialect_options.header.IsSetByUser() ||
 		     (options.dialect_options.header.IsSetByUser() && options.dialect_options.header.GetValue()))) {
 			// This means we have more than one row, hence we can use the first row to detect if we have a header
-			row_idx = 1;
+			start_idx_detection = 1;
 		}
 		// First line where we start our type detection
-		const idx_t start_idx_detection = row_idx;
-
-		for (idx_t col_idx = 0; col_idx < data_chunk.ColumnCount(); col_idx++) {
-			auto &cur_vector = data_chunk.data[col_idx];
-			D_ASSERT(cur_vector.GetVectorType() == VectorType::FLAT_VECTOR);
-			D_ASSERT(cur_vector.GetType() == LogicalType::VARCHAR);
-			auto vector_data = FlatVector::GetData<string_t>(cur_vector);
-			auto null_mask = FlatVector::Validity(cur_vector);
-			auto &col_type_candidates = info_sql_types_candidates[col_idx];
-			for (row_idx = start_idx_detection; row_idx < chunk_size; row_idx++) {
-				// col_type_candidates can't be empty since anything in a CSV file should at least be a string
-				// and we validate utf-8 compatibility when creating the type
-				D_ASSERT(!col_type_candidates.empty());
-				auto cur_top_candidate = col_type_candidates.back();
-				// try cast from string to sql_type
-				while (col_type_candidates.size() > 1) {
-					const auto &sql_type = col_type_candidates.back();
-					// try formatting for date types if the user did not specify one and it starts with numeric
-					// values.
-					string separator;
-					// If Value is not Null, Has a numeric date format, and the current investigated candidate is
-					// either a timestamp or a date
-					if (null_mask.RowIsValid(row_idx) && StartsWithNumericDate(separator, vector_data[row_idx]) &&
-					    (col_type_candidates.back().id() == LogicalTypeId::TIMESTAMP ||
-					     col_type_candidates.back().id() == LogicalTypeId::DATE)) {
-						DetectDateAndTimeStampFormats(candidate->GetStateMachine(), sql_type, separator,
-						                              vector_data[row_idx]);
-					}
-					// try cast from string to sql_type
-					if (sql_type == LogicalType::VARCHAR) {
-						// Nothing to convert it to
-						continue;
-					}
-					if (CanYouCastIt(vector_data[row_idx], sql_type, sniffing_state_machine.dialect_options,
-					                 !null_mask.RowIsValid(row_idx),
-					                 sniffing_state_machine.options.decimal_separator[0])) {
-						break;
-					} else {
-						if (row_idx != start_idx_detection && cur_top_candidate == LogicalType::BOOLEAN) {
-							// If we thought this was a boolean value (i.e., T,F, True, False) and it is not, we
-							// immediately pop to varchar.
-							while (col_type_candidates.back() != LogicalType::VARCHAR) {
-								col_type_candidates.pop_back();
-							}
-							break;
-						}
-						col_type_candidates.pop_back();
-					}
-				}
-			}
-		}
+		SniffTypes(data_chunk, sniffing_state_machine, info_sql_types_candidates, start_idx_detection);
 
+		// Count the number of varchar columns
 		idx_t varchar_cols = 0;
-
 		for (idx_t col = 0; col < info_sql_types_candidates.size(); col++) {
 			auto &col_type_candidates = info_sql_types_candidates[col];
 			// check number of varchar columns
@@ -413,9 +451,10 @@ void CSVSniffer::DetectTypes() {
 
 		// it's good if the dialect creates more non-varchar columns, but only if we sacrifice < 30% of
 		// best_num_cols.
-		if (varchar_cols<min_varchar_cols &&static_cast<double>(info_sql_types_candidates.size())>(
-		        static_cast<double>(max_columns_found) * 0.7) &&
-		    (!options.ignore_errors.GetValue() || candidate->error_handler->errors.size() < min_errors)) {
+		if (!best_candidate ||
+		    (varchar_cols<min_varchar_cols &&static_cast<double>(info_sql_types_candidates.size())>(
+		         static_cast<double>(max_columns_found) * 0.7) &&
+		     (!options.ignore_errors.GetValue() || candidate->error_handler->errors.size() < min_errors))) {
 			min_errors = candidate->error_handler->errors.size();
 			best_header_row.clear();
 			// we have a new best_options candidate
@@ -441,8 +480,9 @@ void CSVSniffer::DetectTypes() {
 		}
 	}
 	if (!best_candidate) {
-		auto error = CSVError::SniffingError(options.file_path);
-		error_handler->Error(error, true);
+		DialectCandidates dialect_candidates(options.dialect_options.state_machine_options);
+		auto error = CSVError::SniffingError(options, dialect_candidates.Print());
+		error_handler->Error(error);
 	}
 	// Assert that it's all good at this point.
 	D_ASSERT(best_candidate && !best_format_candidates.empty());
