@@ -659,7 +659,7 @@ void Linenoise::EditBackspace() {
 	}
 }
 
-static bool IsSpace(char c) {
+bool Linenoise::IsSpace(char c) {
 	switch (c) {
 	case ' ':
 	case '\r':
@@ -1521,9 +1521,9 @@ bool Linenoise::EditBufferWithEditor(const char *editor) {
 #endif
 	string temporary_file_name;
 #ifndef WIN32
-	temporary_file_name = string(tmpdir) + "/duckdb.edit." + std::to_string(getpid());
+	temporary_file_name = string(tmpdir) + "/duckdb.edit." + std::to_string(getpid()) + ".sql";
 #else
-	temporary_file_name = string(tmpdir) + "duckdb.edit." + std::to_string(getpid());
+	temporary_file_name = string(tmpdir) + "duckdb.edit." + std::to_string(getpid()) + ".sql";
 #endif
 
 	FILE *f = fopen(temporary_file_name.c_str(), "w+");
