@@ -491,7 +491,7 @@ def test_jsonlines(shell):
     result = test.run()
     result.check_stdout('{"42":42,"43":43}')
 
-def test_nested_jsonlines(shell):
+def test_nested_jsonlines(shell, json_extension):
     test = (
         ShellTest(shell)
         .statement(".mode jsonlines")
@@ -612,17 +612,23 @@ def test_mode_html(shell):
         .statement("SELECT NULL, 42, 'fourty-two', 42.0;")
     )
     result = test.run()
-    result.check_stdout('<TD>fourty-two</TD>')
+    result.check_stdout('<td>fourty-two</td>')
 
 # Original comment: FIXME sqlite3_column_blob
 def test_mode_insert(shell):
     test = (
         ShellTest(shell)
         .statement(".mode insert")
-        .statement("SELECT NULL, 42, 'fourty-two', 42.0;")
+        .statement("SELECT NULL, 42, 'fourty-two', 42.0, 3.14, 2.71;")
     )
     result = test.run()
     result.check_stdout('fourty-two')
+    result.check_stdout('3.14')
+    result.check_stdout('2.71')
+    result.check_not_exist('3.140000')
+    result.check_not_exist('2.709999')
+    result.check_not_exist('3.139999')
+    result.check_not_exist('2.710000')
 
 def test_mode_line(shell):
     test = (
